searchData={"items":[{"type":"module","title":"Gestalt","doc":"Provides a wrapper for `Application.get_env/3` and `System.get_env/1`, where configuration\ncan be overridden on a per-process basis. This allows asynchronous tests to change\nconfiguration on the fly without altering global state for other tests.","ref":"Gestalt.html"},{"type":"module","title":"Usage - Gestalt","doc":"In `test_helper.exs`, add the following:\n\n    {:ok, _} = Gestalt.start()\n\nIn runtime code, where one would use `Application.get_env/3`,\n\n    value = Application.get_env(:my_module, :my_config)\n\ninstead the following could be used:\n\n    value = Gestalt.get_config(:my_module, :my_config, self())\n\nIn runtime code, where one would use `System.get_env/1`,\n\n    value = System.get_env(\"VARIABLE_NAME\")\n\ninstead the following could be used:\n\n    value = Gestalt.get_env(\"VARIABLE_NAME\", self())","ref":"Gestalt.html#module-usage"},{"type":"module","title":"Overriding values in tests - Gestalt","doc":"The value of Gestalt comes from its ability to change configuration and/or environment\nin a way that only effects the current process. For instance, if code behaves differently\ndepending on configuration, then a test that uses `Application.put_env/4` to verify its\neffect will change global state for other asynchronously-running tests.\n\nTo change Application configuration, use the following:\n\n    Gestalt.replace_config(:my_module, :my_value, \"some value\", self())\n\nTo change System environment, use the following:\n\n    Gestalt.replace_env(\"VARIABLE_NAME\", \"some value\", self())","ref":"Gestalt.html#module-overriding-values-in-tests"},{"type":"module","title":"Caveats - Gestalt","doc":"Gestalt does not try to be too smart about merging overrides with existing configuration.\nIf an override is set for the current pid, then all config and env values required by the\nruntime code must be specifically set.\n\nAlso, note that Gestalt is a runtime configuration library. Values used by module variables\nare evaluated at compile time, not at runtime.","ref":"Gestalt.html#module-caveats"},{"type":"function","title":"Gestalt.copy/3","doc":"Copies Gestalt overrides from one pid to another. If no overrides have been defined,\nreturns `nil`.","ref":"Gestalt.html#copy/3"},{"type":"function","title":"Gestalt.copy!/3","doc":"Copies Gestalt overrides from one pid to another. If no overrides have been defined,\nraises a RuntimeError.","ref":"Gestalt.html#copy!/3"},{"type":"function","title":"Gestalt.get_config/4","doc":"Gets configuration either from Application, or from the running agent.","ref":"Gestalt.html#get_config/4"},{"type":"function","title":"Examples - Gestalt.get_config/4","doc":"iex> {:ok, _pid} = Gestalt.start()\n    iex> Application.put_env(:module_name, :key_name, true)\n    iex> Gestalt.get_config(:module_name, :key_name, self())\n    true\n    iex> Gestalt.replace_config(:module_name, :key_name, false, self())\n    :ok\n    iex> Gestalt.get_config(:module_name, :key_name, self())\n    false","ref":"Gestalt.html#get_config/4-examples"},{"type":"function","title":"Gestalt.get_env/3","doc":"Gets environment variables either from System, or from the running agent.","ref":"Gestalt.html#get_env/3"},{"type":"function","title":"Examples - Gestalt.get_env/3","doc":"iex> {:ok, _pid} = Gestalt.start()\n    iex> System.put_env(\"VARIABLE_FROM_ENV\", \"value set from env\")\n    iex> Gestalt.get_env(\"VARIABLE_FROM_ENV\", self())\n    \"value set from env\"\n    iex> Gestalt.replace_env(\"VARIABLE_FROM_ENV\", \"no longer from env\", self())\n    :ok\n    iex> Gestalt.get_env(\"VARIABLE_FROM_ENV\", self())\n    \"no longer from env\"","ref":"Gestalt.html#get_env/3-examples"},{"type":"function","title":"Gestalt.replace_config/5","doc":"Sets an override for the provided pid, effecting the behavior of `get_config/4`.","ref":"Gestalt.html#replace_config/5"},{"type":"function","title":"Gestalt.replace_env/4","doc":"Sets an override for the provided pid, effecting the behavior of `get_env/3`.","ref":"Gestalt.html#replace_env/4"},{"type":"function","title":"Gestalt.start/1","doc":"Starts an agent for storing override values. If an agent is already running, it\nis returned.","ref":"Gestalt.html#start/1"},{"type":"function","title":"Examples - Gestalt.start/1","doc":"iex> {:ok, pid} = Gestalt.start()\n    iex> is_pid(pid)\n    true\n    iex> {:ok, other_pid} = Gestalt.start()\n    iex> pid == other_pid\n    true","ref":"Gestalt.html#start/1-examples"},{"type":"type","title":"Gestalt.agent/0","doc":"","ref":"Gestalt.html#t:agent/0"},{"type":"module","title":"Gestalt.Macros","doc":"Provides macros that execute Gestalt code only in the :test Mix environment.\nIn other environments, Gestalt is compiled out and either Application or\nSystem is used.","ref":"Gestalt.Macros.html"},{"type":"module","title":"Usage - Gestalt.Macros","doc":"defmodule MyApp.Config do\n      use Gestalt\n\n      def config_value(),\n        do: gestalt_config(:my_app, :config, self())\n\n      def env_value(),\n        do: gestalt_env(\"ENVIRONMENT_VARIABLE\", self())\n    end","ref":"Gestalt.Macros.html#module-usage"},{"type":"macro","title":"Gestalt.Macros.gestalt_config/3","doc":"","ref":"Gestalt.Macros.html#gestalt_config/3"},{"type":"macro","title":"Gestalt.Macros.gestalt_env/2","doc":"","ref":"Gestalt.Macros.html#gestalt_env/2"},{"type":"extras","title":"Overview","doc":"Overview\n========\n\nGestalt serves as a wrapper for `Application.get_env/3` and `System.get_env/1`. It provides a mechanism for setting\nprocess-specific overrides to application configuration and system variables, primarily to ease asynchronous testing of\nbehaviors dependent on specific values.\n\nGestalt should be used for getting runtime configuration, and not in places where configuration is compiled into\nmodules.","ref":"overview.html"},{"type":"extras","title":"Asynchronous Testing - Overview","doc":"Assuming your project sets configuration in `config.exs`, some elixir code will use `Application.get_env/3` to access\nits values.\n\n```elixir\nuse Mix.Config\n\nconfig :my_project, :enable_new_feature, true\n```\n\nThis could be accessed at runtime using a config module:\n\n```elixir\ndefmodule Project.Config do\n  def enable_new_feature? do\n    Application.get_env(:my_project, :enable_new_feature)\n  end\nend\n```\n\nAnd tests could be written:\n\n```elixir\ndefmodule Project.ConfigTest do\n  use ExUnit.Case, async: true\n\n  alias Project.Config\n\n  describe \"enable_new_feature?/0\" do\n    test \"when :enable_new_feature is true, it is true\" do\n      :ok = Application.put_env(:my_project, :enable_new_feature, true)\n      assert Config.enable_new_feature?()\n    end\n\n    test \"when :enable_new_feature is false, it is false\" do\n      :ok = Application.put_env(:my_project, :enable_new_feature, false)\n      refute Config.enable_new_feature?()\n    end\n  end\nend\n```\n\nNow there is a problem. Because these tests are marked `async: true`, there will be times that they will run\nconcurrently. Since application env is global, `Application.put_env/4` will effect everything else in the runtime. If\ncontroller tests or acceptance tests assert on user-facing behavior related to the configuration, then the two tests\nshown above may cause those to fail randomly and non-deterministically.\n\nThe same problem occurs with `System.get_env/1`. Code may behave differently in the presence of a specific environment\nvariable. For instance, optionally initializing a library depending on whether or not an authentication token is\npresent. `System.put_env/2` can be used to update values for tests, leading to more non-deterministic test failures.\n\n```elixir\ndefmodule Project.Config do\n  #...\n\n  def enable_monitoring_lib? do\n    case monitoring_auth_token do\n      nil -> false\n      _ -> true\n    end\n  end\n\n  def monitoring_auth_token do\n    System.get_env(\"AUTH_TOKEN\")\n  end\nend\n```\n\n```elixir\ndefmodule Project.ConfigTest do\n  #...\n\n  describe \"enable_monitoring_lib?/0\" do\n    test \"when AUTH_TOKEN is present, it is true\" do\n      :ok = System.put_env(\"AUTH_TOKEN\", \"abc123\")\n      assert Config.enable_monitoring_lib?()\n    end\n\n    test \"when AUTH_TOKEN is not present, it is false\" do\n      :ok = System.delete_env(\"AUTH_TOKEN\")\n      refute Config.enable_monitoring_lib?()\n    end\n  end\nend\n```\n\nOne solution would be to set `async: false` for all tests that depend upon configuration. Another would be to rewrite\nthe `Config` functions such that `Application` or `System` could be injected. The former would work for testing at the\nacceptance level. The second would not, without jumping through many more hoops of dependency injection.","ref":"overview.html#asynchronous-testing"},{"type":"extras","title":"Pid-specific Overrides - Overview","doc":"Gestalt solves this problem in a different fashion, by starting an `Agent` to store override values for specific pids.\nInitialization of the agent can be done in `test/test_helpers.exs`, for instance:\n\n```elixir\n{:ok, _agent} = Gestalt.start()\n```\n\nNow the config module can be rewritten as follows:\n\n```elixir\ndefmodule Project.Config do\n  def enable_new_feature? do\n    Gestalt.get_config(:my_project, :enable_new_feature, self())\n  end\n\n  def enable_monitoring_lib? do\n    case monitoring_auth_token do\n      nil -> false\n      _ -> true\n    end\n  end\n\n  def monitoring_auth_token do\n    Gestalt.get_env(\"AUTH_TOKEN\", self())\n  end\nend\n```\n\nBy default, when there is no agent running or when there are no overrides for the current pid, `Gestalt.get_config/4`\nfalls back to `Application.get_env/3` and `Gestalt.get_env/2` falls back to `System.get_env/1`. For purposes of clarity\nand to remind us that Gestalt overrides are pid-specific, the pid arguments are not optional. Gestalt functions do take\nan extra optional argument, which is the agent name.\n\nNow our tests can be rewritten as follows to use `Gestalt.replace_config/5` and `Gestalt.replace_env/4`:\n\n```elixir\ndefmodule Project.ConfigTest do\n  use ExUnit.Case, async: true\n\n  alias Project.Config\n\n  describe \"enable_new_feature?/0\" do\n    test \"when :enable_new_feature is true, it is true\" do\n      :ok = Gestalt.replace_config(:my_project, :enable_new_feature, true, self())\n      assert Config.enable_new_feature?()\n    end\n\n    test \"when :enable_new_feature is false, it is false\" do\n      :ok = Gestalt.replace_config(:my_project, :enable_new_feature, false, self())\n      refute Config.enable_new_feature?()\n    end\n  end\n\n  describe \"enable_monitoring_lib?/0\" do\n    test \"when AUTH_TOKEN is present, it is true\" do\n      :ok = Gestalt.replace_env(\"AUTH_TOKEN\", \"abc123\", self())\n      assert Config.enable_monitoring_lib?()\n    end\n\n    test \"when AUTH_TOKEN is not present, it is false\" do\n      :ok = Gestalt.replace_env(\"AUTH_TOKEN\", nil, self())\n      refute Config.enable_monitoring_lib?()\n    end\n  end\nend\n```\n\nNote that `self()` can be used in both the code and the tests, because the code is running in the same pid as the test.\nIn most cases, this will be safe. In some few cases, the code might be running in a separate process from the test, in\nwhich case `replace_config` and `replace_env` should use the pid of the running code.","ref":"overview.html#pid-specific-overrides"},{"type":"extras","title":"Runtime vs. Compile-time - Overview","doc":"Gestalt can be used to override values in the runtime. A common pattern in Elixir dependency injection is to use\napplication config to set module variables. This happens at compile time, making it impossible for Gestalt to\nprovide overrides.","ref":"overview.html#runtime-vs-compile-time"},{"type":"extras","title":"Gestalt macros for testing - Overview","doc":"For cases where runtime configuration should only be overridden during testing, Gestalt provides macros that compile\nto `Application` and `System` in non-test environments.\n\n```elixir\ndefmodule Project.Config do\n  use Gestalt\n\n  def enable_new_feature? do\n    gestalt_config(:my_project, :enable_new_feature, self())\n  end\n\n  def enable_monitoring_lib? do\n    case monitoring_auth_token do\n      nil -> false\n      _ -> true\n    end\n  end\n\n  def monitoring_auth_token do\n    gestalt_env(\"AUTH_TOKEN\", self())\n  end\nend\n```","ref":"overview.html#gestalt-macros-for-testing"},{"type":"extras","title":"Acceptance tests - Overview","doc":"Acceptance tests often involve multiple processes. Wallaby, for instance, starts a Phoenix server, which processes\nrequests in separate processes. In this case, `self()` in the context of the test process will be different from\n`self()` in the server process.\n\nThis can by solved with the `Gestalt.copy/2` function, in a test plug.\n\nIf the `test` Mix env includes `test/support` in its elixir paths, then a plug can be written in\n`test/support/test/plug/gestalt.ex`\n\n```elixir\ndefmodule ProjectWeb.Test.Plug.Gestalt do\n  alias Plug.Conn\n\n  def init(_), do: nil\n\n  def call(conn, _opts),\n    do:\n      conn\n      |> extract_metadata()\n      |> copy_overrides(conn)\n\n  defp extract_metadata(%Conn{} = conn),\n    do:\n      conn\n      |> Conn.get_req_header(\"user-agent\")\n      |> List.first()\n      |> Phoenix.Ecto.SQL.Sandbox.decode_metadata()\n\n  defp copy_overrides(%{gestalt_pid: gestalt_pid}, conn) do\n    Gestalt.copy(gestalt_pid, self())\n    conn\n  end\n\n  defp copy_overrides(_metadata, conn), do: conn\nend\n```\n\nThis can then be added to the Phoenix Endpoint:\n\n```elixir\ndefmodule ProjectWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_project\n\n  if Application.get_env(:my_project, :sql_sandbox) do\n    plug(Phoenix.Ecto.SQL.Sandbox)\n    plug(ProjectWeb.Test.Plug.Gestalt)\n  end\n  \n  # ....\nend\n```\n\nIn the `CaseTemplate` used for your acceptance tests, the following can then be configured (if using Wallaby):\n\n```elixir\nsetup tags do\n  :ok = Ecto.Adapters.SQL.Sandbox.checkout(Project.Repo)\n  unless tags[:async], do: Ecto.Adapters.SQL.Sandbox.mode(Project.Repo, {:shared, self()})\n\n  # Add the :gestalt_pid test process pid to the metadata being passed through each acceptance request header\n  metadata = Phoenix.Ecto.SQL.Sandbox.metadata_for(Project.Repo, self()) |> Map.put(:gestalt_pid, self())\n  {:ok, session} = Wallaby.start_session(metadata: metadata)\n  {:ok, session: session}\nend\n```\n\nNow, any overrides configured for the test process will be copied onto the server process pid, and\n`Gestalt.get_config/3` or `Gestalt.get_env/2` will have your overrides available.","ref":"overview.html#acceptance-tests"},{"type":"extras","title":"LiveView tests - Overview","doc":"LiveView tests can be configured to share the Gestalt configuration via `Phoenix.LiveView.on_mount/1`.\nIn the test, the current pid should be added to the `conn`'s session.\n\n```elixir\nPhoenix.ConnTest.init_test_session(conn, %{gestalt_pid: self()})\n{:ok, _live, _html} = live(conn, \"/\")\n```\n\nThe LiveView can run an `on_mount` handler to copy configuration for the socket's pid.\n\n```elixir\ndefmodule Web.Gestalt do\n  def on_mount(:copy_gestalt_config, _params, %{\"gestalt_pid\" => gestalt_pid}, socket) do\n    Gestalt.copy(gestalt_pid, self())\n    {:cont, socket}\n  end\n\n  def on_mount(:copy_gestalt_config, _params, _session, socket) do\n    {:cont, socket}\n  end\nend\n```\n\n```elixir\nmodule Web.MyLive do\n  use Web, :live_view\n\n  on_mount {Web.Gestalt, :copy_gestalt_config}\n\n  def render(assigns) do\n    ~H\" \"\n  end\n\n  def mount(_params, _session, socket), do: {:ok, socket}\nend\n```","ref":"overview.html#liveview-tests"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}